import { useStore } from '../context.jsx'
import { changeCase } from '../util.jsx'

import { Icon } from './Icon.jsx'

import { AdminPanel } from '../pages/Admin.jsx'
import { ApiKeys } from '../pages/ApiKeys.jsx'
import { Login } from '../pages/Login.jsx'
import { Dashboard } from '../pages/Dashboard.jsx'
import { Buckets } from '../pages/Buckets.jsx'
import { Bucket } from '../pages/Bucket.jsx'

export const MainContent = () => {
    const { setSidebarVisible, route, user } = useStore()

    const Route =
        {
            index: Dashboard,
            login: Login,
            admin: AdminPanel,
            apiKeys: ApiKeys,
            buckets: Buckets,
            bucket: Bucket,
        }[route.id] || (() => <>Still nothing here</>)

    return (
        <div class={`main-content route-${changeCase('camel', 'dash', route.id)}`}>
            <header>
                <div class="group">
                    {user === 'admin' && (
                        <div
                            class="button icon flat"
                            onClick={() => setSidebarVisible(value => !value)}
                        >
                            <Icon name="menu" />
                        </div>
                    )}
                    <div class="label">
                        {route.id === 'bucket' ? (
                            <>
                                Bucket / <code>{route.params.bucket}</code>
                            </>
                        ) : (
                            route.name
                        )}
                    </div>
                </div>
                {user === 'admin' ? (
                    <div class="label">{user}</div>
                ) : (
                    route.id !== 'login' && (
                        <div class="group">
                            <a class="button flat" href="/login">
                                Login
                            </a>
                        </div>
                    )
                )}
            </header>
            <div class="content">
                <Route />
            </div>
        </div>
    )
}
