import { useEffect, useState } from 'preact/hooks'
import { Icon } from './Icon.jsx'
import { useStore } from '../context.jsx'
import { useRemoteState } from '../util.jsx'
import { useToasts } from './Toasts.jsx'

const SidebarDropdownItem = ({ icon, label, children }) => {
    const [collapsed, setCollapsed] = useState(false)

    return (
        <div class="item dropdown">
            <div class="label" onClick={() => setCollapsed(x => !x)}>
                <div class="row">
                    <div class="row-group">
                        <span class="icon">
                            <Icon name={icon} />
                        </span>
                        <span>{label}</span>
                    </div>
                    <div class="row-group">
                        <span class="icon">
                            <Icon name={collapsed ? 'expand_more' : 'expand_less'} />
                        </span>
                    </div>
                </div>
            </div>
            <div class={collapsed ? 'children collapsed' : 'children'}>{children}</div>
        </div>
    )
}

export const Sidebar = () => {
    const { sidebarVisibile, user } = useStore()

    const [showToast] = useToasts()

    const [buckets, err] = useRemoteState('/api/buckets', [])
    useEffect(() => {
        if (err) {
            showToast(`Errore "${err}"`)
        }
    }, [err])

    return (
        user === 'admin' && (
            <div class={'sidebar' + (sidebarVisibile ? '' : ' collapsed')}>
                <div class="logo">Storage</div>
                <div class="items">
                    <div class="item single">
                        <a href="/" class="label">
                            <div class="row">
                                <div class="row-group">
                                    <Icon name="dashboard" />
                                    <div class="text">Dashboard</div>
                                </div>
                            </div>
                        </a>
                    </div>
                    <div class="item single">
                        <a href="/admin" class="label">
                            <div class="row">
                                <div class="row-group">
                                    <Icon name="security" />
                                    <div class="text">Admin Panel</div>
                                </div>
                            </div>
                        </a>
                    </div>
                    <div class="item single">
                        <a href="/api-keys" class="label">
                            <div class="row">
                                <div class="row-group">
                                    <Icon name="key" />
                                    <div class="text">API Keys</div>
                                </div>
                            </div>
                        </a>
                    </div>
                    <hr />
                    <div class="item single">
                        <a href="/buckets" class="label">
                            <div class="row">
                                <div class="row-group">
                                    <Icon name="database" />
                                    <div class="text">Buckets</div>
                                </div>
                            </div>
                        </a>
                    </div>
                    {buckets.map(bucket => (
                        <div class="item" style="margin-left: 1.75rem;">
                            <a href={'/buckets/' + bucket} class="label">
                                <div class="row">
                                    <div class="row-group">
                                        <Icon name="view_list" />
                                        <div>{bucket}</div>
                                    </div>
                                </div>
                            </a>
                        </div>
                    ))}
                    {/* <hr /> */}
                </div>
            </div>
        )
    )
}
