import { createContext } from 'preact'
import { useContext, useState } from 'preact/hooks'
import { Icon } from './Icon.jsx'

let globalToastId = 0

const ToastContext = createContext([])

export const ToastMessage = ({ icon, children }) => {
    return (
        <div class="row">
            {icon && <Icon name={icon} />}
            <p>{children}</p>
        </div>
    )
}

export const ToastProvider = ({ children }) => {
    const [toasts, setToasts] = useState([])

    const removeToast = uid => {
        setToasts(toasts => toasts.filter(t => t.uid !== uid))
    }

    return (
        <ToastContext.Provider value={{ toasts, setToasts }}>
            {children}
            <div class="toasts">
                {toasts.map(({ uid, removed, message }) => (
                    <div class={'toast' + (removed ? ' removed' : '')}>
                        <p>{message}</p>
                        <div class="toast-close" onClick={() => removeToast(uid)}>
                            <Icon name="close" />
                        </div>
                    </div>
                ))}
            </div>
        </ToastContext.Provider>
    )
}

export const useToasts = () => {
    const { setToasts } = useContext(ToastContext)

    const showToast = (message, { duration } = {}) => {
        duration ??= 3000

        const toast = { uid: globalToastId++, removed: false, message }

        setTimeout(() => {
            setToasts(toasts => toasts.map(t => (t === toast ? { ...t, removed: true } : t)))
        }, duration)

        setTimeout(() => {
            setToasts(toasts => toasts.filter(t => t !== toast))
        }, duration + 1000)

        setToasts(ts => [...ts.slice(-5), toast])
    }

    const clearToasts = () => {
        setToasts([])
    }

    return [showToast, clearToasts]
}
