import { useEffect, useRef, useState } from 'preact/hooks'
import { hashCode } from '../../util.jsx'

let pieChartPiecesSeed = 934635493

export const PieChart = ({ parts, labels, total, totalLabel }) => {
    const [seed] = useState(() => pieChartPiecesSeed++)

    parts ??= [1]
    labels ??= parts
    totalLabel ??= ''

    const canvasRef = useRef()

    useEffect(() => {
        if (canvasRef.current) {
            const $canvas = canvasRef.current

            const size = Math.min($canvas.offsetWidth, $canvas.offsetHeight)

            $canvas.width = size * 2
            $canvas.height = size * 2

            $canvas.style.width = `${size}px`
            $canvas.style.height = `${size}px`

            const width = $canvas.width / 2
            const height = $canvas.height / 2

            const g = $canvas.getContext('2d')
            g.resetTransform()
            g.clearRect(0, 0, width * 2, height * 2)

            g.scale(2, 2)
            g.translate(width / 2, height / 2)

            g.font = `bold 16px 'Open Sans'`
            g.textAlign = 'center'
            g.textBaseline = 'middle'

            total = total || parts.reduce((acc, p) => acc + p)
            const anglesAndLabel = parts.map((p, i) => [(p / total) * 2 * Math.PI, labels[i] || ''])

            g.fillStyle = `#ededed`
            g.beginPath()
            g.ellipse(0, 0, width * 0.5 * 0.8, width * 0.5 * 0.8, 0, 0, 2 * Math.PI)
            g.fill()

            g.strokeStyle = `#00000044`
            g.beginPath()
            g.ellipse(0, 0, width * 0.5 * 0.8, width * 0.5 * 0.8, 0, 0, 2 * Math.PI)
            g.stroke()

            let acc = 0
            anglesAndLabel.forEach(([angle, label], i) => {
                g.fillStyle = `hsl(${((hashCode(`${i} ${seed}`) % 0xff) * 360) / 0xff}, 80%, 65%)`
                g.beginPath()
                g.moveTo(0, 0)
                g.arc(0, 0, width * 0.5 * 0.8, acc - 0.5 * Math.PI, acc + angle - 0.5 * Math.PI)
                g.fill()

                g.strokeStyle = `#00000044`
                g.beginPath()
                g.moveTo(0, 0)
                g.arc(0, 0, width * 0.5 * 0.8, acc - 0.5 * Math.PI, acc + angle - 0.5 * Math.PI)
                g.lineTo(0, 0)
                g.stroke()

                g.fillStyle = `hsl(${((hashCode(`${i} ${seed}`) % 0xff) * 360) / 0xff}, 80%, 15%)`
                g.fillText(
                    label,
                    Math.cos(acc + angle / 2 - 0.5 * Math.PI) * (width * 0.5 * 0.9125),
                    Math.sin(acc + angle / 2 - 0.5 * Math.PI) * (width * 0.5 * 0.9125)
                )

                acc += angle
            })

            if (totalLabel && Math.abs(2 * Math.PI - acc) > Math.PI / 32) {
                const totalLabelAngle = acc + (2 * Math.PI - acc) / 2

                g.fillStyle = '#333'
                g.fillText(
                    totalLabel,
                    Math.cos(totalLabelAngle - 0.5 * Math.PI) * width * 0.5 * 0.5,
                    Math.sin(totalLabelAngle - 0.5 * Math.PI) * width * 0.5 * 0.5
                )
            }
        }
    }, [canvasRef, parts, labels])

    return <canvas ref={canvasRef}></canvas>
}
