import { byteSizeToString, useRemoteState } from '../../util.jsx'
import { PieChart } from '../charts/PieChart.jsx'
import { Icon } from '../Icon.jsx'
import { ToastMessage, useToasts } from '../Toasts.jsx'

export const DiskUsage = ({ disk, diskLabel }) => {
    const [showToast] = useToasts()
    const [output, err, refreshOutput] = useRemoteState(
        `/api/monitor/status?script=${encodeURIComponent(`disk-usage ${disk}`)}`,
        '100\n0'
    )

    const [total, used] = output.split('\n').map(s => parseInt(s))

    const title = (
        <>
            Spazio Utilizzato &bull; <code>{diskLabel || disk}</code>
        </>
    )

    return (
        <>
            <div class="title">
                <div class="row">
                    <div class="row-group">
                        <p>{title}</p>
                    </div>
                    <div class="row-group">
                        <button
                            class="icon"
                            onClick={() => {
                                refreshOutput()
                                showToast(
                                    <ToastMessage icon="info">Aggiornato "{title}"</ToastMessage>
                                )
                            }}
                        >
                            <Icon name="refresh" />
                        </button>
                    </div>
                </div>
            </div>
            <div class="content">
                {err ? (
                    <p>Errore "{err}"</p>
                ) : (
                    <>
                        <PieChart
                            parts={[used]}
                            labels={[byteSizeToString(used)]}
                            total={total}
                            totalLabel={''}
                        />
                        <p class="center">
                            Spazio totale <strong>{byteSizeToString(total)}</strong>
                        </p>
                    </>
                )}
            </div>
        </>
    )
}
