import { useEffect, useState } from 'preact/hooks'
import { Icon } from '../components/Icon.jsx'
import { useToasts } from '../components/Toasts.jsx'
import { useStore } from '../context.jsx'

export const ApiKeys = () => {
    const [showToast] = useToasts()

    const [keys, setKeys] = useState([])

    useEffect(() => {
        fetch('/api/api-keys')
            .then(res => res.json())
            .then(value => setKeys(value))
            .catch(e => console.error(e))
    }, [])

    const createKey = async () => {
        const res = await fetch('/api/api-keys', { method: 'POST' })
        if (!res.ok) {
            showToast('Errore!')
        }

        const key = await res.json()

        setKeys(keys => [...keys, key])
    }

    const removeKey = async key => {
        const res = await fetch('/api/api-keys/' + key, { method: 'DELETE' })

        if (res.ok) {
            showToast('Chiave rimossa')
            setKeys(keys => keys.filter(k => k !== key))
        } else {
            showToast('Errore!')
        }
    }

    const copyKey = async key => {
        await navigator.clipboard.writeText(key)
        showToast(
            <>
                Copiata chiave <code>{key.substring(0, 6)}</code>... negli appunti
            </>
        )
    }

    return (
        <>
            <div class="panel">
                <div class="row">
                    <div class="row-group">
                        <div class="title">API Keys</div>
                    </div>
                    <div class="row-group">
                        <button class="icon" onClick={() => createKey()}>
                            <Icon name="add" />
                        </button>
                    </div>
                </div>
                <div class="table">
                    <div class="cells" style="grid-template-columns: 1fr auto;">
                        {/* Header */}
                        <div class="header">Chiave</div>
                        <div class="header">Azioni</div>
                        {/* Rows */}
                        {keys.map(key => {
                            return (
                                <>
                                    <div class="name">
                                        <button class="flat" onClick={() => copyKey(key)}>
                                            {key}
                                        </button>
                                    </div>
                                    <div class="actions">
                                        <button
                                            class="icon delete flat"
                                            onClick={() => removeKey(key)}
                                        >
                                            <Icon name="delete" />
                                        </button>
                                    </div>
                                </>
                            )
                        })}
                    </div>
                </div>
            </div>
        </>
    )
}
