import { useEffect, useRef, useState } from 'preact/hooks'
import { Icon } from '../components/Icon.jsx'
import { ToastMessage, useToasts } from '../components/Toasts.jsx'
import { useStore } from '../context.jsx'
import { useRemoteState } from '../util.jsx'

export const Bucket = () => {
    const {
        route: {
            params: { bucket },
        },
    } = useStore()

    const [showToast] = useToasts()

    const [bucketObjects, err, updateBucketObjects] = useRemoteState(`/api/buckets/${bucket}`, [])
    useEffect(() => {
        if (err) {
            showToast(`Errore "${err}"`)
        }
    }, [err])

    const fileInputRef = useRef(null)

    const uploadFile = async () => {
        const formData = new FormData()

        formData.append('file', fileInputRef.current.files[0])

        showToast(<ToastMessage icon="cloud_upload">Caricando il file...</ToastMessage>)

        const res = await fetch(`/api/buckets/${bucket}`, {
            method: 'POST',
            body: formData,
        })

        if (res.ok) {
            const { id } = await res.json()

            showToast(
                <ToastMessage icon="info">
                    File caricato con ID: <code>{id}</code>
                </ToastMessage>,
                { duration: 10e3 }
            )

            await updateBucketObjects()
        }
    }

    const deleteObject = async id => {
        if (!confirm(`Cancellare l'oggetto con id ${id} da questo bucket?`)) {
            showToast(<ToastMessage icon="info">Operazione annullata</ToastMessage>)
            return
        }

        const res = await fetch(`/api/buckets/${bucket}/${id}`, { method: 'DELETE' })

        if (res.ok) {
            showToast(<ToastMessage icon="info">Oggetto rimosso</ToastMessage>)

            await updateBucketObjects()
        } else {
            showToast(
                <ToastMessage icon="error">Errore del server "{await res.text()}"</ToastMessage>
            )
        }
    }

    return (
        <>
            <div class="panel">
                <div class="row">
                    <div class="row-group">
                        <div class="title">
                            Bucket / <code>{bucket}</code>
                        </div>
                    </div>
                    <div class="row-group">
                        <button class="icon" onClick={() => fileInputRef.current.click()}>
                            <Icon name="file_upload" />
                        </button>
                        <input
                            type="file"
                            ref={fileInputRef}
                            onChange={() => uploadFile()}
                            style="display: none;"
                        />
                    </div>
                </div>
                <div class="table">
                    <div class="cells">
                        {/* Header */}
                        <div class="header">Objects</div>
                        <div class="header">Actions</div>
                        {/* Rows */}
                        {bucketObjects.map(key => (
                            <>
                                <div class="name">
                                    <code>{key}</code>
                                </div>
                                <div class="options">
                                    <a
                                        class="button icon flat"
                                        href={`/api/buckets/${bucket}/${key}`}
                                    >
                                        <Icon name="file_download" />
                                    </a>
                                    <button
                                        class="icon delete flat"
                                        onClick={() => deleteObject(key)}
                                    >
                                        <Icon name="delete" />
                                    </button>
                                </div>
                            </>
                        ))}
                    </div>
                </div>
            </div>
        </>
    )
}
