import { useEffect, useState } from 'preact/hooks'
import { Icon } from '../components/Icon.jsx'
import { useToasts } from '../components/Toasts.jsx'
import { useRemoteState } from '../util.jsx'

export const Buckets = () => {
    const [showToast] = useToasts()

    const [buckets, err, updateBuckets] = useRemoteState('/api/buckets', [])
    useEffect(() => {
        if (err) {
            showToast(`Errore "${err}"`)
        }
    }, [err])

    const [allBucketSettings, setAllBucketSettings] = useState({})

    const updateBucketSettings = async bucket => {
        const res = await fetch(`/api/buckets/${bucket}/settings`)
        const value = await res.json()

        setAllBucketSettings(s => ({ ...s, [bucket]: value }))
    }

    useEffect(() => {
        for (const bucket of buckets) {
            updateBucketSettings(bucket)
        }
    }, [buckets])

    const [newBucketName, setNewBucketName] = useState('')
    const [newBucketSettings, setNewBucketSettings] = useState('')

    const createBucket = async () => {
        const { path } = JSON.parse(newBucketSettings || '{"path":""}')

        const res = await fetch(`/api/buckets/`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                bucket: newBucketName,
                path,
            }),
        })

        if (res.ok) {
            await updateBuckets()
        } else {
            showToast(`Errore: ${await res.text()}`)
        }
    }

    const deleteBucket = async bucket => {
        showToast('TODO: Not implemented')
    }

    return (
        <>
            <div class="panel">
                <div class="row">
                    <div class="row-group">
                        <div class="title">Buckets</div>
                    </div>
                </div>
                <div class="table">
                    <div class="cells">
                        {/* Header */}
                        <div class="header">Bucket</div>
                        <div class="header">Settings</div>
                        <div class="header">Actions</div>
                        {/* Rows */}
                        {buckets.map(bucket => (
                            <>
                                <div class="name">
                                    <a href={`/buckets/${bucket}`}>
                                        <code>{bucket}</code>
                                    </a>
                                </div>
                                <div class="settings">
                                    <input
                                        readonly
                                        class="mono"
                                        type="text"
                                        value={JSON.stringify(allBucketSettings[bucket])}
                                    />
                                </div>
                                <div class="actions">
                                    <button
                                        class="icon delete flat"
                                        onClick={() => deleteBucket(bucket)}
                                    >
                                        <Icon name="delete" />
                                    </button>
                                </div>
                            </>
                        ))}
                    </div>
                </div>
            </div>

            <form action="/api/buckets" method="POST" autocomplete="off">
                <div class="fill title">Nuovo Bucket</div>

                <label for="new-bucket-name">Name</label>
                <input
                    type="text"
                    name="bucket"
                    id="new-bucket-name"
                    placeholder="new-bucket"
                    value={newBucketName}
                    onInput={e => setNewBucketName(e.target.value)}
                />
                <label for="new-bucket-settings">Settings</label>
                <input
                    type="text"
                    name="settings"
                    id="new-bucket-settings"
                    placeholder={`{"path":"${newBucketName || 'new-bucket'}/"}`}
                    value={newBucketSettings}
                    onInput={e => setNewBucketSettings(e.target.value)}
                />

                <div class="fill center">
                    <button type="button" onClick={() => createBucket()}>
                        Crea Bucket
                    </button>
                </div>
            </form>
        </>
    )
}
