import { createContext, toChildArray } from 'preact'
import { useCallback, useContext, useEffect, useRef, useState } from 'preact/hooks'

export function hashCode(s) {
    s = s.toString() + "seed iniziale dell'hash"

    let hash = 0

    if (s.length === 0) return hash

    for (let i = 0; i < s.length; i++) {
        const chr = s.charCodeAt(i)
        hash = (hash << 5) - hash + chr
        hash |= 0
    }

    return Math.abs(hash)
}

export function useUser(onLoginStateChanged) {
    const [user, setUser] = useState(null)

    useEffect(() => {
        fetch('/api/current-user')
            .then(res => res.json())
            .then(value => {
                setUser(value)
                onLoginStateChanged(value)
            })
            .catch(e => console.error(e))
    }, [])

    return user
}

//
// Hooks
//

// Router

function matchPattern(pattern, url) {
    const r = `^${pattern.replace(/:([a-zA-Z0-9\_\-]+)/g, '(?<$1>[^\\/\\?]+?)')}$`
    return new RegExp(r).exec(url)
}

export const useRouter = routes => {
    for (const [id, route] of Object.entries(routes)) {
        const m = matchPattern(route.pattern, location.pathname)
        if (m) {
            return [id, route, m.groups]
        }
    }

    return ['unknown', { id: 'unknown' }, {}]
}

// useRemoteState

export function useRemoteState(url, initialValue = null) {
    const [error, setError] = useState(null)
    const [value, setValue] = useState(initialValue)

    const refresh = async () => {
        try {
            const res = await fetch(url)
            if (!res.ok) {
                setError(await res.text())
                return
            }

            setValue(await res.json())
        } catch (err) {
            setError(err)
        }
    }

    useEffect(() => {
        refresh()
    }, [])

    return [value, error, refresh]
}

// Toggle CSS Class

// export function useClasses(classes) {
//     const elementRef = useRef(null)

//     if (Array.isArray(classes)) {
//         classes = Object.fromEntries(classes.filter(Boolean).map(className => [className, true]))
//     }

//     const updateClasses = () => {
//         if (elementRef.current) {
//             const $el = elementRef.current

//             console.log($el.classList)
//             console.log('Applying classes:', classes)

//             for (const [className, present] of Object.entries(classes)) {
//                 if (present) {
//                     if (!$el.classList.contains(className)) {
//                         console.log('Adding class:', className, present)
//                         $el.classList.add(className)
//                     }
//                 } else {
//                     if ($el.classList.contains(className)) {
//                         console.log('Removing class:', className, present)
//                         $el.classList.remove(className)
//                     }
//                 }
//             }
//         }
//     }

//     useEffect(() => {
//         updateClasses()
//     })

//     return [elementRef]
// }

//
// Change Case Utility
//

const fromCaseMap = {
    camel: s => s.split(/(?=[A-Z])/),
    // ...
}
const toCaseMap = {
    dash: parts => parts.map(p => p.toLowerCase()).join('-'),
    // ...
}

export function changeCase(from, to, s) {
    return toCaseMap[to](fromCaseMap[from](s))
}

//
// Disk space conversions
//

const byteUnits = {
    K: 1024,
    M: 1024 ** 2,
    G: 1024 ** 3,
    T: 1024 ** 4,
}

export function byteSizeToString(size) {
    let i = 0
    while (size >= 450) {
        size /= 1024
        i++
    }

    return `${size.toFixed(2)}${Object.keys(byteUnits)[i]}B`
}
